import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_database/models/enums/nui_database_data_type.dart';

class NUIDBField<T> extends NUIEntField<T>{
  NUIDBField({@required String name, @required NUIEntType type, @required EntFieldSetter<T> setter, dynamic additionalFormat, @required EntFieldGetter<T> getter, this.primaryKey: false, this.ignore:false}) : super(name: name, type: type, setter: setter, getter: getter, additionalFormat: additionalFormat);
  final bool ignore;
  final bool primaryKey;

  String _getNativeType(){
    return type.nativeType();
  }

  String getCreateQuery(){
    return "$name ${_getNativeType()}${primaryKey ? " PRIMARY KEY" : ""}";
  }

  NUIDBFieldTracker toFieldTracker() => NUIDBFieldTracker(name: name, type: type.value(), primaryKey: primaryKey);

  bool structureChanged(NUIDBFieldTracker tracker){
    final samePrimaryKey = (tracker.primaryKey ?? false) == (primaryKey ?? false);
    final sameType = match(tracker.type, type.value());
    final sameName = match(tracker.name, name);
    return !samePrimaryKey || !sameName || !sameType;
  }
}

class NUIDBFieldQuery<T> extends NUIDBField<T>{
  NUIDBFieldQuery({@required String name, @required NUIEntType type, bool primaryKey:false}): super(name: name, type: type, primaryKey: primaryKey, setter: null, getter: null);
}

class NUIDBFieldTracker extends NUIEnt<NUIDBFieldTracker>{
  NUIDBFieldTracker.empty();
  NUIDBFieldTracker({this.name, this.type, this.primaryKey: false});
  String name;
  String type;
  bool primaryKey;

  String _getNativeType(){
    return valueOfEntType(type).nativeType();
  }

  String getCreateQuery(){
    return "$name ${_getNativeType()}${primaryKey ? " PRIMARY KEY" : ""}";
  }

  @override
  String toString(){
    return "$name $type${primaryKey ? " PRIMARY KEY" : ""}";
  }

  @override
  NUIEntMapper<NUIDBFieldTracker> mapper() => NUIDBFieldTrackerEntity();
}

class NUIDBFieldTrackerEntity extends NUIEntMapper<NUIDBFieldTracker>{
  @override
  String entityName() => "NUIDBFieldTracker";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIDBFieldTracker>(name: "name", type: NUIEntType.STRING, setter: (data, value) => data.name = value, getter: (data) => data.name),
      NUIEntField<NUIDBFieldTracker>(name: "type", type: NUIEntType.STRING, setter: (data, value) => data.type = value, getter: (data) => data.type),
      NUIEntField<NUIDBFieldTracker>(name: "primaryKey", type: NUIEntType.BOOLEAN, setter: (data, value) => data.primaryKey = value, getter: (data) => data.primaryKey),
    ];
  }

  @override
  NUIDBFieldTracker newInstance() => NUIDBFieldTracker.empty();
}


